#ifndef lint
static const char rcsid[] = "$Id: test-tw2.c,v 1.1.1.1 1996/06/30 15:50:53 mtp Exp $";
#endif
/****************************************************************************
 *    
 *  COPYRIGHT NOTICE
 *    
 *  Copyright (C) 1995, 1996 Michael T. Peterson
 *  This file is part of the PCthreads (tm) multithreading library
 *  package.
 *    
 *  The source files and libraries constituting the PCthreads (tm) package
 *  are free software; you can redistribute them and/or modify them under
 *  the terms of the GNU Library General Public License as published by the Free
 *  Software Foundation; either version 2 of the License, or (at your
 *  option) any later version.
 *    
 *  The PCthreads (tm) package is distributed in the hope that it will
 *  be useful, but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 *  Library General Public License for more details.
 *    
 *  You should have received a copy of the GNU Library General Public
 *  License along with this library (see the file COPYING.LIB); if not,
 *  write to the Free Software Foundation, Inc., 675 Mass Ave, Cambridge,
 *  MA 02139, USA.
 *    
 *  To report bugs, bug fixes, or provide feedback you may contact the
 *  author, Michael T. Peterson, at either of the two email addresses listed
 *  below:
 *    
 *             mtp@big.aa.net (preferred)
 *             mtp@zso.dec.com
 *    
 *    
 *  Michael T. Peterson
 *  Issaquah, WA
 *  29 January, 1996
 ****************************************************************************
 * $Log: test-tw2.c,v $
 * Revision 1.1.1.1  1996/06/30 15:50:53  mtp
 * DCE threads for Linux V1.0
 *
 ****************************************************************************
 */
#include <stdlib.h>
#include <stdio.h>
#include <pthread.h>
#include <exc_handling.h>
#include <utils.h>

static pthread_t *th;
static struct timespec *tspec;
static pthread_mutex_t mu;
static pthread_cond_t cv;

static void 
cv_timer( struct timespec *ts )
{
    volatile int exit_status = SUCCESS;
    int done = FALSE, status;

    status = pthread_mutex_lock( &mu );
    TH_CHECK(status, "pthread_mutex_lock()" );

    TRY
        while( !done )
        {
            status = pthread_cond_timedwait( &cv, &mu, ts );
            TH_CHECK( status, "pthread_cond_timedwait()" );

            done = TRUE;
        }

    CATCH_ALL

        /*
         * Received a cancel. By construction we were cancelled within the
         * pthread_cond_timedwait() and therefore no longer have the mutex.
         * Accordingly, we must relock the mutex and proceed.
         */
        exit_status = PTHREAD_CANCELED;
        status = pthread_mutex_lock( &mu );
        TH_CHECK(status, "pthread_mutex_unlock()" );

    ENDTRY

    status = pthread_mutex_unlock( &mu );
    TH_CHECK(status, "pthread_mutex_unlock()" );

    pthread_exit( (void *) exit_status );
}

int 
main( int argc, char *argv[] )
{
    int i, st, exit_status = -1, thread_count = 1;
    struct timespec tmo;
    struct timeval now;

    if( argc == 2 )
        thread_count = atoi( argv[1] );

    st = pthread_mutex_init( &mu, NULL );
        TH_CHECK( st, "pthread_mutex_init()");

    st = pthread_cond_init( &cv, NULL );
    TH_CHECK( st, "pthread_cond_init()");

    th = malloc( thread_count * sizeof( pthread_t ) );
    tspec = malloc( thread_count * sizeof( struct timespec ) );

    for(i = 0; i < thread_count; i++ )
    {
        gettimeofday( &now, NULL );
        tspec[i].tv_sec = now.tv_sec + 1;
        tspec[i].tv_nsec = rand() % 999999;

        st =  create_joinable( &th[i], (thread_proc_t) cv_timer, &tspec[i] );
        TH_CHECK( st, "create_joinable()");
    }

    tmo.tv_sec = 0; tmo.tv_nsec = 500000; /* 1 seconds */

    for(i = 0; i < thread_count; i++ )
    {    
        pthread_delay_np( &tmo );
        st = pthread_cancel( th[i] );
        TH_CHECK(st, "pthread_cancel()");
    }

    for(i = 0; i < thread_count; i++ )
    {
        st = pthread_join( th[i], (void **) &exit_status );
        TH_CHECK( st, "pthread_join()");

        if( exit_status == PTHREAD_CANCELED )
            printf("%x cancelled!\n", th[i] );
        else
            printf( "%x terminated with status = %d!\n", th[i], exit_status );
    }

    free( th );
    free( tspec );
    st = pthread_cond_destroy( &cv );
    TH_CHECK( st, "pthread_cond_destroy()");

    st = pthread_mutex_destroy( &mu );
    TH_CHECK( st, "pthread_mutex_destroy()");

    return( EXIT_SUCCESS );
}
