#ifndef lint
static const char rcsid[] = "$Id: test1.c,v 1.1.1.1 1996/06/30 15:50:54 mtp Exp $";
#endif
/****************************************************************************
 *    
 *  COPYRIGHT NOTICE
 *    
 *  Copyright (C) 1995, 1996 Michael T. Peterson
 *  This file is part of the PCthreads (tm) multithreading library
 *  package.
 *    
 *  The source files and libraries constituting the PCthreads (tm) package
 *  are free software; you can redistribute them and/or modify them under
 *  the terms of the GNU Library General Public License as published by the Free
 *  Software Foundation; either version 2 of the License, or (at your
 *  option) any later version.
 *    
 *  The PCthreads (tm) package is distributed in the hope that it will
 *  be useful, but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 *  Library General Public License for more details.
 *    
 *  You should have received a copy of the GNU Library General Public
 *  License along with this library (see the file COPYING.LIB); if not,
 *  write to the Free Software Foundation, Inc., 675 Mass Ave, Cambridge,
 *  MA 02139, USA.
 *    
 *  To report bugs, bug fixes, or provide feedback you may contact the
 *  author, Michael T. Peterson, at either of the two email addresses listed
 *  below:
 *    
 *             mtp@big.aa.net (preferred)
 *             mtp@zso.dec.com
 *    
 *    
 *  Michael T. Peterson
 *  Issaquah, WA
 *  29 January, 1996
 ****************************************************************************
 * $Log: test1.c,v $
 * Revision 1.1.1.1  1996/06/30 15:50:54  mtp
 * DCE threads for Linux V1.0
 *
 ****************************************************************************
 */
#include <stdlib.h>
#include <stdio.h>
#include <pthread.h>
#include "utils.h"

void lock_mu( pthread_mutex_t *mu )
{
   int st;

   printf_r("Child attempting to lock mutex\n");
   st = pthread_mutex_lock( mu );
   CHECK(st, "pthread_mutex_lock()");
   printf_r("...Child has locked mutex\n");

   st = pthread_mutex_unlock( mu );
   CHECK(st, "pthread_mutex_unlock()");
   printf_r("...Child has unlocked mutex\n");

   pthread_exit( (void *) SUCCESS );
}

void
init_mu( pthread_mutex_t *mu )
{
   int st;

   st = pthread_mutex_init( mu, NULL );
   CHECK( st, "pthread_mutex_init()");
}

void
destroy_mu( pthread_mutex_t *mu )
{
   int st;

   st = pthread_mutex_destroy( mu );
   CHECK(st, "pthread_mutex_destroy()");
}

static pthread_mutex_t mutex;
static pthread_t th;

int 
main( int argc, char *argv[] )
{
   int st, exit_status;
   pthread_attr_t th_attr;

   init_mu( &mutex );

   /*
    *  --  Set the child's thread attributes so that the main() thread
    *      can join it.
    */
   (void) pthread_attr_init( &th_attr );
   (void) pthread_attr_setdetachstate( &th_attr, PTHREAD_CREATE_JOINABLE );

   /*
    *  --  Now, lock the mutex, and create the child thread.  The child
    *      will attempt to lock the same mutex, and block until
    *      the mutex is released.
    */
   st = pthread_mutex_lock( &mutex );
   CHECK(st, "pthread_mutex_lock()");
   printf_r("Main has locked the mutex\n");

   /*
    *  --  This is the second thread that should block when it attempts to
    *      lock mu.
    */
   st = pthread_create( &th, &th_attr, (thread_proc_t)lock_mu, &mutex );
   CHECK(st, "pthread_create()");
   pthread_yield( NULL );

   /*
    *  --  Unlock the mutex
    */
   st = pthread_mutex_unlock( &mutex );
   CHECK(st, "pthread_mutex_unlock()");
   printf_r("Main has unlocked the mutex\n");

   st = pthread_join( th, (void **) &exit_status );
   CHECK(st, "pthread_join()");

   if( exit_status != SUCCESS )
       printf_r("Failed to join\n");

   destroy_mu( &mutex );
   return( EXIT_SUCCESS );
}
